# coding=utf-8

"""
Copyright 2015 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import absolute_import

__all__ = ['DataStore', 'LoadZone',
           'UserScenario', 'UserScenarioValidation']

import json

from .exceptions import CoercionError, ResponseParseError
from .fields import (
    DataStoreListField, DateTimeField, Field, IntegerField,
    UnicodeField, BooleanField)
from pprint import pformat


class Resource(object):
    """All API resources derive from this base class."""

    fields = {}

    def __init__(self, client, **kwargs):
        super(Resource, self).__setattr__('_fields', {})
        self.client = client
        self._set_fields(kwargs)

    def __getattr__(self, name):
        fields = self.__class__.fields
        if name in fields:
            return self._fields[name].value
        raise AttributeError("'%s' object has no field '%s'"
                             % (self.__class__.__name__, name))

    def __setattr__(self, name, value):
        fields = self.__class__.fields
        if name in fields:
            self._fields[name].value = value
        super(Resource, self).__setattr__(name, value)

    def __repr__(self):
        return "<%s>\n%s" % (self.__class__.__name__, pformat(self._fields))

    @classmethod
    def _path(cls, resource_id=None, action=None):
        if resource_id is not None:
            if action:
                return '%s/%s/%s' % (cls.resource_name, str(resource_id),
                                     action)
            return '%s/%s' % (cls.resource_name, str(resource_id))
        return cls.resource_name

    def _set_fields(self, data):
        fields = self.__class__.fields
        for k, f in fields.items():
            if isinstance(f, tuple):
                fun, opts = f
                self._fields[k] = fun(data.get(k, fun.default()), options=opts)
            else:
                self._fields[k] = f(data.get(k, f.default()))


class GetMixin(object):
    @classmethod
    def get(cls, client, resource_id):
        response = client.get(cls._path(resource_id))
        try:
            instance = cls(client)
            name = cls.resource_response_object_name
            r = response.json()
            r_obj = r.get(name)
            instance._set_fields(r_obj)
            return instance
        except CoercionError as e:
            raise ResponseParseError(e)

    def sync(self):
        response = self.client.get(self.__class__._path(self.id))
        try:
            name = self.resource_response_object_name
            r = response.json()
            r_obj = r.get(name)
            self._set_fields(r_obj)
        except CoercionError as e:
            raise ResponseParseError(e)


class CreateMixin(object):
    create_content_type = 'application/json'

    @classmethod
    def create(cls, client, data, file_object=None):
        headers = None if file_object else {'Content-Type':
                                            cls.create_content_type}
        if not file_object and isinstance(data, dict):
            data = json.dumps(data)
        response = client.post(cls._path(), headers=headers, data=data,
                               file_object=file_object)
        try:
            instance = cls(client)
            name = cls.resource_response_object_name
            r = response.json()
            r_obj = r.get(name)
            instance._set_fields(r_obj)
            return instance
        except CoercionError as e:
            raise ResponseParseError(e)


class DeleteMixin(object):
    def delete(self):
        self.client.delete(self.__class__._path(resource_id=self.id))

    @classmethod
    def delete_with_id(cls, client, resource_id):
        client.delete(cls._path(resource_id=resource_id))


class UpdateMixin(object):

    @classmethod
    def update(cls, client, resource_id, data=None, file_object=None):
        if data:
            if not file_object and isinstance(data, str):
                data = json.loads(data)
        response = client.put(cls._path(resource_id=resource_id),
                              headers='', data=data, file_object=file_object)
        try:
            instance = cls(client)
            name = cls.resource_response_object_name
            r = response.json()
            r_obj = r.get(name)
            instance._set_fields(r_obj)
            return instance
        except CoercionError as e:
            raise ResponseParseError(e)


class ListMixin(object):

    @classmethod
    def list(cls, client, resource_id=None, project_id=None):
        if project_id:
            cls.project_id = project_id
        path = cls._path(resource_id=resource_id)
        response = client.get(path)
        objects = []
        try:
            name = cls.resource_response_objects_name
            r = response.json()
            r_obj = r.get(name)
            for obj in r_obj:
                instance = cls(client)
                instance._set_fields(obj)
                objects.append(instance)
            return objects
        except CoercionError as e:
            raise ResponseParseError(e)


class DataStore(Resource, ListMixin, GetMixin, CreateMixin, UpdateMixin, DeleteMixin):
    resource_name = 'data-stores'
    resource_response_object_name = 'data_store'
    resource_response_objects_name = 'data_stores'
    project_id = None

    fields = {
        'id': IntegerField,
        'name': UnicodeField,
        'status': IntegerField,
        'rows': IntegerField,
        'created': DateTimeField,
        'belongs_to_user': BooleanField,
        'project_id': IntegerField,
        'public_url': UnicodeField,
        'converted': BooleanField,
    }

    # Data store conversion status codes
    STATUS_QUEUED = 0
    STATUS_CONVERTING = 1
    STATUS_FINISHED = 2
    STATUS_FAILED = 3

    def has_conversion_finished(self):
        """Check whether data store conversion has finished or not.

        Args:
            client: API client instance.

        Returns:
            True if data store conversion has completed, otherwise False.

        """

        self.sync()

        if self.status in [DataStore.STATUS_FINISHED, DataStore.STATUS_FAILED]:
            return True
        return False

    @classmethod
    def _path(cls, resource_id=None):
        if cls.project_id:
            return '{0}?project_id={1}'.format(cls.resource_name, cls.project_id)
        return super(DataStore, cls)._path(resource_id)

    @classmethod
    def status_code_to_text(cls, status_code):
        if DataStore.STATUS_QUEUED == status_code:
            return 'queued'
        elif DataStore.STATUS_CONVERTING == status_code:
            return 'converting'
        elif DataStore.STATUS_FINISHED == status_code:
            return 'finished'
        elif DataStore.STATUS_FAILED == status_code:
            return 'failed'
        return 'unknown'


class LoadZone(Resource, ListMixin):
    resource_name = 'load-zones'
    fields = {
        'id': UnicodeField,
        'name': UnicodeField,
        'city': UnicodeField,
        'country': UnicodeField,
        'vendor': UnicodeField
    }

    # Aggregate load zones
    AGGREGATE_WORLD = 'world'

    # Amazon load zones
    AMAZON_US_ASHBURN = 'amazon:us:ashburn'
    AMAZON_US_PALOALTO = 'amazon:us:palo alto'
    AMAZON_IE_DUBLIN = 'amazon:ie:dublin'
    AMAZON_SG_SINGAPORE = 'amazon:sg:singapore'
    AMAZON_JP_TOKYO = 'amazon:jp:tokyo'
    AMAZON_US_PORTLAND = 'amazon:us:portland'
    AMAZON_BR_SAOPAULO = 'amazon:br:são paulo'
    AMAZON_AU_SYDNEY = 'amazon:au:sydney'

    # Rackspace load zones
    RACKSPACE_US_CHICAGO = 'rackspace:us:chicago'
    RACKSPACE_US_DALLAS = 'rackspace:us:dallas'
    RACKSPACE_UK_LONDON = 'rackspace:uk:chicago'
    RACKSPACE_AU_SYDNEY = 'rackspace:au:sydney'

    # Mapping of load zone names to IDs.
    NAME_TO_ID_MAP = {
        AGGREGATE_WORLD: 1,
        AMAZON_US_ASHBURN: 11,
        AMAZON_US_PALOALTO: 12,
        AMAZON_IE_DUBLIN: 13,
        AMAZON_SG_SINGAPORE: 14,
        AMAZON_JP_TOKYO: 15,
        AMAZON_US_PORTLAND: 22,
        AMAZON_BR_SAOPAULO: 23,
        AMAZON_AU_SYDNEY: 25,
        RACKSPACE_US_CHICAGO: 26,
        RACKSPACE_US_DALLAS: 27,
        RACKSPACE_UK_LONDON: 28,
        RACKSPACE_AU_SYDNEY: 29
    }

    @classmethod
    def name_to_id(cls, name):
        try:
            return cls.NAME_TO_ID_MAP[name]
        except KeyError:
            raise ValueError("There's no load zone with name '%s'" % name)


class UserScenario(Resource, ListMixin, GetMixin, CreateMixin, DeleteMixin,
                   UpdateMixin):
    resource_name = 'user-scenarios'
    resource_response_object_name = 'user_scenario'
    resource_response_objects_name = 'user_scenarios'
    project_id = None
    fields = {
        'name': (UnicodeField, Field.SERIALIZE),
        'script': (UnicodeField, Field.SERIALIZE),
        'data_store_ids': (DataStoreListField, Field.SERIALIZE),
        'created': DateTimeField,
        'updated': DateTimeField,
        'last_validation_id': IntegerField,
        'last_validated': DateTimeField,
        'lines': IntegerField,
        'data_store_counter': IntegerField,
        'last_validation_error': (UnicodeField, Field.SERIALIZE),
        'belongs_to_user': BooleanField,
        'project_id': IntegerField,
        'id': IntegerField
    }

    @classmethod
    def _path(cls, resource_id=None):
        if cls.project_id:
            return '{0}?project_id={1}'.format(cls.resource_name, cls.project_id)
        return super(UserScenario, cls)._path(resource_id)

    def validate(self):
        return self.client.create_user_scenario_validation(
            {'user_scenario_id': self.id})

    def update_scenario(self, data):
        fields = self.fields
        for k, f in fields.items():
            if not data.get(k):
                data[k] = getattr(self, k)
        return super(UserScenario, self).update(self.client, self.id, data)


class UserScenarioValidationResult(Resource, ListMixin):
    resource_name = 'validations'
    resource_response_object_name = 'user_scenario_validation_results'
    resource_response_objects_name = 'user_scenario_validation_results'

    fields = {
        'user_scenario_validation_id': IntegerField,
        'type': IntegerField,
        'offset': IntegerField,
        'level': (UnicodeField, Field.SERIALIZE),
        'message': (UnicodeField, Field.SERIALIZE),
        'timestamp': DateTimeField
    }

    @classmethod
    def _path(cls, resource_id):
        return '{0}/{1}/results'.format(cls.resource_name, resource_id)


class UserScenarioValidation(Resource, GetMixin, CreateMixin):
    resource_name = 'validations'
    resource_response_object_name = 'user_scenario_validation'
    fields = {
        'id': IntegerField,
        'user_scenario_id': IntegerField,
        'status': IntegerField,
        'status_text': UnicodeField,
        'queued': DateTimeField,
        'started': DateTimeField,
        'ended': DateTimeField
    }

    # Validation status codes
    STATUS_QUEUED = 0
    STATUS_INITIALIZING = 1
    STATUS_RUNNING = 2
    STATUS_FINISHED = 3
    STATUS_FAILED = 4

    def is_done(self):
        """Check whether validation is done or not.

        Returns:
            True if validation has completed, otherwise False.
        """
        if self.status in [UserScenarioValidation.STATUS_FINISHED,
                           UserScenarioValidation.STATUS_FAILED]:
            return True
        return False

    @classmethod
    def status_code_to_text(cls, status_code):
        if UserScenarioValidation.STATUS_QUEUED == status_code:
            return 'queued'
        elif UserScenarioValidation.STATUS_INITIALIZING == status_code:
            return 'initializing'
        elif UserScenarioValidation.STATUS_RUNNING == status_code:
            return 'running'
        elif UserScenarioValidation.STATUS_FINISHED == status_code:
            return 'finished'
        elif UserScenarioValidation.STATUS_FAILED == status_code:
            return 'failed'
        return 'unknown'


class Organization(Resource, ListMixin):
    resource_name = 'organizations'
    resource_response_object_name = 'organization'
    resource_response_objects_name = 'organizations'
    fields = {
        'name': (UnicodeField, Field.SERIALIZE),
        'id': IntegerField
    }


class OrganizationProject(Resource, ListMixin):
    resource_name = 'organizations'
    resource_response_object_name = 'project'
    resource_response_objects_name = 'projects'
    fields = {
        'name': (UnicodeField, Field.SERIALIZE),
        'id': IntegerField
    }

    @classmethod
    def _path(cls, resource_id):
        return '{0}/{1}/projects'.format(cls.resource_name, resource_id)



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__all__ = ['UTC']

from datetime import timedelta, tzinfo


_ZERO = timedelta(0)


def is_dict_different(d1, d2, epsilon=0.00000000001):
    s1 = set(d1.keys())
    s2 = set(d2.keys())
    intersect = s1.intersection(s2)
    added = s1 - intersect
    removed = s2 - intersect
    changed = []
    for o in intersect:
        if isinstance(d2[o], float):
            if abs(d2[o] - d1[o]) > epsilon:
                changed.append(o)
        elif d2[o] != d1[o]:
            changed.append(o)
    return (0 < len(added) or 0 < len(removed) or 0 < len(set(changed)))


class UTC(tzinfo):
    def utcoffset(self, dt):
        return _ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return _ZERO



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

from loadimpact3.utils import is_dict_different, UTC


class TestUtilsFunctions(unittest.TestCase):
    def test_is_dict_different_false(self):
        d1 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        d2 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        self.assertFalse(is_dict_different(d1, d2))

    def test_is_dict_different_true_int(self):
        d1 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        d2 = {
            'int': 1,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        self.assertTrue(is_dict_different(d1, d2))

    def test_is_dict_different_true_string(self):
        d1 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        d2 = {
            'int': 0,
            'string': "I'm a string2",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        self.assertTrue(is_dict_different(d1, d2))

    def test_is_dict_different_true_float(self):
        d1 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        d2 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.123456,
            'dict': {'key': 'value'}
        }
        self.assertTrue(is_dict_different(d1, d2))

    def test_is_dict_different_false_float_epsilon(self):
        d1 = {
            'float': 0.123457
        }
        d2 = {
            'float': 0.123456
        }
        self.assertFalse(is_dict_different(d1, d2, epsilon=0.00001))

    def test_is_dict_different_true_float_epsilon(self):
        d1 = {
            'float': 0.123457
        }
        d2 = {
            'float': 0.123456
        }
        self.assertTrue(is_dict_different(d1, d2, epsilon=0.0000001))

    def test_is_dict_different_true_dict(self):
        d1 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value'}
        }
        d2 = {
            'int': 0,
            'string': "I'm a string",
            'float': 0.12345,
            'dict': {'key': 'value2'}
        }
        self.assertTrue(is_dict_different(d1, d2))


class TestUtilsUTC(unittest.TestCase):
    def setUp(self):
        self.tz = UTC()

    def assertTimeDeltaZero(self, td):
        self.assertEqual(td.days, 0)
        self.assertEqual(td.seconds, 0)
        self.assertEqual(td.microseconds, 0)

    def test_utcoffset(self):
        self.assertTimeDeltaZero(self.tz.utcoffset(None))

    def test_tzname(self):
        self.assertEqual(self.tz.tzname(None), 'UTC')

    def test_dst(self):
        self.assertTimeDeltaZero(self.tz.dst(None))



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__all__ = ['ApiError', 'MissingApiTokenError', 'ResponseParseError',
           'ConnectionError', 'TimeoutError', 'HTTPError', 'ClientError',
           'BadRequestError', 'UnauthorizedError', 'ForbiddenError',
           'NotFoundError', 'MethodNotAllowedError', 'ConflictError',
           'GoneError', 'RateLimitError', 'ServerError']


class ApiError(Exception):
    """All Load Impact API exceptions derive from this class."""


class MissingApiTokenError(ApiError):
    """Raised when AI client is created and an API token can't be found."""


class CoercionError(ApiError):
    """Raised when a resource field value coercion fails."""


class ResponseParseError(ApiError):
    """Raised when parsing of API response fails."""


class ConnectionError(ApiError):
    """Raised when a TCP connection error is encountered."""


class TimeoutError(ApiError):
    """Raised when a TCP connection timeout is encountered."""


class HTTPError(ApiError):
    """All HTTP exception classes derive from this base class."""

    def __init__(self, msg=None, response=None):
        super(HTTPError, self).__init__(msg)
        self.response = response


class ClientError(HTTPError):
    """Raised when 4xx HTTP response code is encountered with no specialized
    exception class.
    """


class BadRequestError(ClientError):
    """Raised when 400 HTTP status code is encountered."""


class UnauthorizedError(ClientError):
    """Raised when 401 HTTP status code is encountered."""


class ForbiddenError(ClientError):
    """Raised when 403 HTTP status code is encountered."""


class NotFoundError(ClientError):
    """Raised when 404 HTTP status code is encountered."""


class MethodNotAllowedError(ClientError):
    """Raised when 405 HTTP status code is encountered."""


class ConflictError(ClientError):
    """Raised when 409 HTTP status code is encountered."""


class GoneError(ClientError):
    """Raised when 410 HTTP status code is encountered."""


class RateLimitError(ClientError):
    """Raised when 427 HTTP status code is encountered."""


class ServerError(HTTPError):
    """Raised when 5xx HTTP response code is encountered with no specialized
    exception class.
    """



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import requests
import unittest

from loadimpact3.clients import ApiTokenClient, Client
from loadimpact3.exceptions import (
    ApiError, BadRequestError, ClientError, ConnectionError,
    ForbiddenError, GoneError, HTTPError, MethodNotAllowedError,
    MissingApiTokenError, NotFoundError, RateLimitError, ServerError,
    TimeoutError, UnauthorizedError)
from loadimpact3.resources import DataStore

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class MockRequestsResponse(object):
    def __init__(self, expecting_list=False, resource_type='object', status_code=200, **kwargs):
        self.expecting_list = expecting_list
        self.url = 'http://example.com/'
        self.status_code = status_code
        self.text = ''
        self.resource_type = resource_type
        self.kwargs = kwargs
        for k, v in kwargs.items():
            setattr(self, k, v)

    def json(self):
        return {self.resource_type: [self.kwargs]} if self.expecting_list else {self.resource_type: self.kwargs}


class MockClient(Client):
    def __init__(self, expecting_list=False, resource_type=None, timeout=Client.default_timeout,
                 **kwargs):
        super(MockClient, self).__init__(timeout=timeout)
        self.expecting_list = expecting_list
        self.resource_type = resource_type
        self.kwargs = kwargs
        self.last_request_method = None
        self.last_request_args = None
        self.last_request_kwargs = None

    def _requests_request(self, method, *args, **kwargs):
        self.last_request_method = method
        self.last_request_args = args
        self.last_request_kwargs = kwargs
        if isinstance(kwargs.get('data'), str):
            self.last_request_kwargs['data'] = json.loads(kwargs['data'])
        if 'raise_exception_cls' in self.kwargs:
            raise self.kwargs.get('raise_exception_cls')
        nkwargs = {}
        if kwargs.get('data'):
            if isinstance(kwargs['data'], dict):
                nkwargs = kwargs['data']
            elif isinstance(kwargs['data'], str):
                nkwargs = json.loads(kwargs['data'])
        return MockRequestsResponse(expecting_list=self.expecting_list, resource_type=self.resource_type,
                                    **nkwargs)


class MockApiTokenClient(ApiTokenClient):
    def __init__(self, api_token=None, **kwargs):
        super(MockApiTokenClient, self).__init__(api_token=api_token, **kwargs)
        self.kwargs = kwargs
        self.last_request_method = None
        self.last_request_args = None
        self.last_request_kwargs = None

    def _requests_request(self, method, *args, **kwargs):
        self.last_request_method = method
        self.last_request_args = args
        self.last_request_kwargs = kwargs
        return MockRequestsResponse()


class MockApiTokenFromEnvClient(MockApiTokenClient):
    def __init__(self, api_token_env=None, **kwargs):
        self.api_token_env = api_token_env
        super(MockApiTokenFromEnvClient, self).__init__(api_token=None,
                                                        **kwargs)

    def _get_api_token_from_environment(self):
        return self.api_token_env


class MockApiTokenFromEnvErrorClient(MockApiTokenClient):
    def _get_api_token_from_environment(self):
        raise KeyError


class TestClientsClient(unittest.TestCase):
    def test_timeout(self):
        timeout = 12345
        client = MockClient(timeout=timeout)
        client.get('some-fake-path')
        self.assertEqual(client.last_request_kwargs['timeout'], timeout)

    def test_create_data_store(self):
        client = MockClient(resource_type='data_store')

        data = {
            'name': 'Test Data Store',
            'project_id': 1,
            'fromline': 0,
            'separator': 'comma',
            'delimeter': 'tab',
            'quotechar': 'double',
            'file': ''
        }
        csv = 'column1,column2,column3'
        data_store = client.create_data_store(data, StringIO(csv))

        self.assertEqual(client.last_request_method, 'post')
        self.assertEqual(client.last_request_kwargs['data']['name'],
                         data['name'])
        self.assertEqual(client.last_request_kwargs['data']['fromline'],
                         data['fromline'])
        self.assertEqual(client.last_request_kwargs['data']['separator'],
                         data['separator'])
        self.assertEqual(client.last_request_kwargs['data']['delimeter'],
                         data['delimeter'])
        self.assertEqual(client.last_request_kwargs['files']['file'].read(),
                         StringIO(csv).read())
        self.assertEqual(data_store.name, data['name'])
        self.assertEqual(data_store.status, DataStore.STATUS_QUEUED)
        self.assertEqual(data_store.rows, 0)

    def test_get_data_store(self):
        client = MockClient(resource_type='data_store')
        data_store = client.get_data_store(1)

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(data_store.id, 0)  # ID = 0 by default
        self.assertEqual(data_store.name, '')
        self.assertEqual(data_store.status, DataStore.STATUS_QUEUED)
        self.assertEqual(data_store.rows, 0)

    def test_list_data_stores(self):
        client = MockClient(resource_type='data_stores', expecting_list=True)
        data_stores = client.list_data_stores(project_id=1)

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(len(data_stores), 1)
        self.assertEqual(data_stores[0].id, 0)  # ID = 0 by default
        self.assertEqual(data_stores[0].name, '')
        self.assertEqual(data_stores[0].status, DataStore.STATUS_QUEUED)
        self.assertEqual(data_stores[0].rows, 0)

    def test_create_user_scenario(self):
        client = MockClient(resource_type='user_scenario')
        data = {
            'name': 'Test User Scenario',
            'script': 'log.info("Hello World!")',
            'project_id': 1
        }
        user_scenario = client.create_user_scenario(data)

        self.assertEqual(client.last_request_method, 'post')
        self.assertEqual(client.last_request_kwargs['data']['name'],
                         data['name'])
        self.assertEqual(client.last_request_kwargs['data']['script'],
                         data['script'])
        self.assertEqual(user_scenario.name, data['name'])
        self.assertEqual(user_scenario.script, data['script'])

    def test_get_user_scenario(self):
        client = MockClient(resource_type='user_scenario')
        user_scenario = client.get_user_scenario(1)

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(user_scenario.id, 0)  # ID = 0 by default
        self.assertEqual(user_scenario.name, '')
        self.assertEqual(user_scenario.script, '')

    def test_list_user_scenarios(self):
        client = MockClient(expecting_list=True, resource_type='user_scenarios')
        user_scenarios = client.list_user_scenarios(project_id=1)

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(len(user_scenarios), 1)
        self.assertEqual(user_scenarios[0].id, 0)  # ID = 0 by default
        self.assertEqual(user_scenarios[0].name, '')
        self.assertEqual(user_scenarios[0].script, '')

    def test_create_user_scenario_validation(self):
        client = MockClient(resource_type='user_scenario_validation')
        data = {
            'user_scenario_id': 1
        }
        validation = client.create_user_scenario_validation(data)

        self.assertEqual(client.last_request_method, 'post')
        self.assertEqual(client.last_request_kwargs['data']['user_scenario_id'],
                         data['user_scenario_id'])
        self.assertEqual(validation.user_scenario_id, data['user_scenario_id'])

    def test_list_organizations(self):
        client = MockClient(resource_type='organizations', expecting_list=True)
        organizations = client.list_organizations()

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(len(organizations), 1)
        self.assertEqual(organizations[0].id, 0)  # ID = 0 by default
        self.assertEqual(organizations[0].name, '')

    def test_list_projects_for_organization(self):
        client = MockClient(resource_type='projects', expecting_list=True)
        projects = client.list_organization_projects(1)

        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(len(projects), 1)
        self.assertEqual(projects[0].id, 0)  # ID = 0 by default
        self.assertEqual(projects[0].name, '')

    def test__check_response_200(self):
        client = MockClient()
        r = MockRequestsResponse(status_code=200, message='OK')
        self.assertEqual(client._check_response(r), r)

    def test__check_response_300(self):
        client = MockClient()
        r = MockRequestsResponse(status_code=300, message='Redirection')
        self.assertEqual(client._check_response(r), r)

    def test__check_response_400(self):
        self._check_response_exception(BadRequestError, 400, 'Bad request')

    def test__check_response_400_no_error_msg(self):
        client = MockClient()
        r = MockRequestsResponse(status_code=400)
        self.assertRaises(BadRequestError, client._check_response, r)

    def test__check_response_401(self):
        self._check_response_exception(UnauthorizedError, 401, 'Unauthorized')

    def test__check_response_402(self):
        self._check_response_exception(ClientError, 402, 'Payment required')

    def test__check_response_403(self):
        self._check_response_exception(ForbiddenError, 403, 'Forbidden')

    def test__check_response_404(self):
        self._check_response_exception(NotFoundError, 404, 'Not found')

    def test__check_response_405(self):
        self._check_response_exception(MethodNotAllowedError, 405,
                                       'Method not allowed')

    def test__check_response_410(self):
        self._check_response_exception(GoneError, 410, 'Gone')

    def test__check_response_427(self):
        self._check_response_exception(RateLimitError, 427, 'Rate limited')

    def test__check_response_500(self):
        self._check_response_exception(ServerError, 500, 'Internal Server Error')

    def test_requests_httperror_exceptions_wrapping(self):
        self._exceptions_wrapping_all_http_verbs(
            requests.exceptions.HTTPError, HTTPError)

    def test_requests_connectionerror_exceptions_wrapping(self):
        self._exceptions_wrapping_all_http_verbs(
            requests.exceptions.ConnectionError, ConnectionError)

    def test_requests_timeouterror_exceptions_wrapping(self):
        self._exceptions_wrapping_all_http_verbs(
            requests.exceptions.Timeout, TimeoutError)

    def test_requests_requestexception_exceptions_wrapping(self):
        self._exceptions_wrapping_all_http_verbs(
            requests.exceptions.RequestException, ApiError)

    def _check_response_exception(self, raise_cls, status_code, message):
        client = MockClient()
        r = MockRequestsResponse(status_code=status_code, message=message)
        self.assertRaises(raise_cls, client._check_response, r)

    def _exceptions_wrapping_all_http_verbs(self, raise_cls, expected_cls):
        client = MockClient(raise_exception_cls=raise_cls)
        self.assertRaises(expected_cls, client.delete, 'some-fake-path')
        self.assertRaises(expected_cls, client.get, 'some-fake-path')
        self.assertRaises(expected_cls, client.post, 'some-fake-path')
        self.assertRaises(expected_cls, client.put, 'some-fake-path')


class TestClientsApiTokenClient(unittest.TestCase):
    def test_missing_api_token_exception(self):
        self.assertRaises(MissingApiTokenError, MockApiTokenFromEnvErrorClient)

    def test_api_token_in_env(self):
        api_token = 'test_token'
        client = MockApiTokenFromEnvClient(api_token_env=api_token)
        client.get('some-fake-path')
        self.assertEqual(client.last_request_kwargs['auth'], (api_token, ''))

    def test_api_token_in_delete_kwarg(self):
        self._check_api_token_in_kwarg('delete')

    def test_api_token_in_get_kwarg(self):
        self._check_api_token_in_kwarg('get')

    def test_api_token_in_post_kwargs(self):
        self._check_api_token_in_kwarg('post')

    def test_api_token_in_put_kwarg(self):
        self._check_api_token_in_kwarg('put')

    def _check_api_token_in_kwarg(self, method):
        api_token = 'test_token'
        client = MockApiTokenClient(api_token=api_token)
        getattr(client, method)('some-fake-path')
        self.assertEqual(client.last_request_kwargs['auth'], (api_token, ''))



# coding=utf-8

"""
Copyright 2015 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

from datetime import datetime
from loadimpact3.clients import Client
from loadimpact3.exceptions import CoercionError
from loadimpact3.fields import (
    Field, DataStoreListField, DateTimeField, IntegerField)
from loadimpact3.resources import Resource
from loadimpact3.utils import UTC


class MockClient(Client):
    def _requests_request(self, method, *args, **kwargs):
        pass


class MockResource(Resource):
    fields = {}

    def __init__(self, client, field_cls, field_value=None):
        self.__class__.fields['field'] = field_cls
        super(MockResource, self).__init__(client, field=field_value)


class TestFieldsField(unittest.TestCase):
    def test_coerce(self):
        self.assertRaises(NotImplementedError, Field.coerce, None)


class TestFieldsDataStoreListField(unittest.TestCase):
    def setUp(self):
        self.client = MockClient()

    def test_coerce(self):
        coerced = DataStoreListField.coerce([1, 2, 3, 4, 5])
        self.assertEqual(coerced, [1, 2, 3, 4, 5])

    def test_coerce_from_list_of_dicts(self):
        coerced = DataStoreListField.coerce([{'id': 6}, {'id': 7}, {'id': 8}])
        self.assertEqual(coerced, [6, 7, 8])

    def test_get(self):
        r = MockResource(self.client, DataStoreListField, [1, 2, 3, 4, 5])
        self.assertEqual(r.field, [1, 2, 3, 4, 5])


class TestFieldsDateTimeField(unittest.TestCase):
    def setUp(self):
        self.now = datetime.utcnow().replace(tzinfo=UTC(), microsecond=0)
        self.client = MockClient()

    def test_coerce(self):
        value = '%s+00:00.' % self.now.strftime(DateTimeField.format)
        coerced = DateTimeField.coerce(value)
        self.assertEqual(coerced, self.now)

    def test_coerce_bad_format(self):
        self.assertRaises(CoercionError, DateTimeField.coerce, '2013-01-01')

    def test_construct_bad_format(self):
        self.assertRaises(CoercionError, MockResource, self.client,
                          DateTimeField, '2013-01-01')

    def test_get(self):
        value = '%s+00:00.' % self.now.strftime(DateTimeField.format)
        r = MockResource(self.client, DateTimeField, value)
        self.assertEqual(r.field, self.now)


class TestFieldsIntegerField(unittest.TestCase):
    def setUp(self):
        self.client = MockClient()

    def test_coerce(self):
        coerced = IntegerField.coerce(1)
        self.assertEqual(coerced, 1)

    def test_coerce_from_string(self):
        coerced = IntegerField.coerce('1')
        self.assertEqual(coerced, 1)

    def test_get(self):
        r = MockResource(self.client, IntegerField, 1)
        self.assertEqual(r.field, 1)



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import absolute_import

__all__ = ['ApiTokenClient']

try:
    import httplib
except ImportError:
    from http import client as httplib

import os
import platform
import requests
from six import raise_from

from .exceptions import (
    ApiError, BadRequestError, ConflictError, ConnectionError, ClientError,
    ForbiddenError, HTTPError, GoneError, MethodNotAllowedError,
    MissingApiTokenError, NotFoundError, RateLimitError, ServerError,
    TimeoutError, UnauthorizedError)
from .resources import (
    DataStore, UserScenario, UserScenarioValidation,
    UserScenarioValidationResult, Organization, OrganizationProject)

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

from .version import __version__


def requests_exceptions_handling(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise_from(ConnectionError(str(e)), None)
        except requests.exceptions.HTTPError as e:
            raise_from(HTTPError(str(e)), None)
        except requests.exceptions.Timeout as e:
            raise TimeoutError(str(e))
        except requests.exceptions.RequestException as e:
            raise ApiError(str(e))
    return wrapper


class Client(object):
    """Base client class handling all communication with the Load Impact REST
    API, using simple API token based authentication."""

    api_base_url = 'https://api.loadimpact.com/v3/'
    default_timeout = 30
    error_classes = {
        400: BadRequestError,
        401: UnauthorizedError,
        403: ForbiddenError,
        404: NotFoundError,
        405: MethodNotAllowedError,
        409: ConflictError,
        410: GoneError,
        427: RateLimitError
    }
    library_versions = "python %s; requests %s" % (platform.python_version(),
                                                   requests.__version__)
    user_agent = "LoadImpactPythonSDK/%s (%s)" % (__version__, library_versions)

    def __init__(self, timeout=default_timeout, debug=False):
        self.timeout = timeout
        if debug:
            httplib.HTTPConnection.debuglevel = 1

    def create_data_store(self, data, file_object):
        return DataStore.create(self, data, file_object=file_object)

    def update_data_store(self, resource_id, data, file_object):
        return DataStore.update(self, resource_id, data, file_object=file_object)

    def get_data_store(self, resource_id):
        return DataStore.get(self, resource_id)

    def list_data_stores(self, project_id):
        return DataStore.list(self, project_id=project_id)

    def create_user_scenario(self, data):
        return UserScenario.create(self, data)

    def get_user_scenario(self, resource_id):
        return UserScenario.get(self, resource_id)

    def list_user_scenarios(self, project_id):
        return UserScenario.list(self, project_id=project_id)

    def create_user_scenario_validation(self, data):
        return UserScenarioValidation.create(self, data)

    def get_user_scenario_validation(self, resource_id):
        return UserScenarioValidation.get(self, resource_id)

    def get_user_scenario_validation_result(self, resource_id):
        return UserScenarioValidationResult.list(self, resource_id)

    def list_organizations(self):
        return Organization.list(self)

    def list_organization_projects(self, org_id):
        return OrganizationProject.list(self, org_id)

    @requests_exceptions_handling
    def delete(self, path, headers=None, params=None):
        """Make a DELETE request to the API.

        Args:
            path: Path of resource URI to where we're making the request.
            headers: Dict of headers to send with request.
            params: Dict with query string parameters.

        Returns:
            A requests response object on success.

        Raises:
            BadRequestError: Request was deemed formatted incorrectly by server.
            UnauthorizedError: API token is incorrect/not valid.
            ForbiddenError: Permission denied.
            APIError: Generic error from requests library.
        """
        url = urljoin(self.__class__.api_base_url, path)
        response = self._request('delete', url, headers=headers, params=params)
        return self._check_response(response)

    @requests_exceptions_handling
    def get(self, path, headers=None, params=None):
        """Make a GET request to the API.

        Args:
            path: Path of resource URI to where we're making the request.
            headers: Dict of headers to send with request.
            params: Dict with query string parameters.

        Returns:
            A requests response object on success.

        Raises:
            BadRequestError: Request was deemed formatted incorrectly by server.
            UnauthorizedError: API token is incorrect/not valid.
            ForbiddenError: Permission denied.
            APIError: Generic error from requests library.
        """
        url = urljoin(self.__class__.api_base_url, path)
        response = self._request('get', url, headers=headers, params=params)
        return self._check_response(response)

    @requests_exceptions_handling
    def post(self, path, headers=None, params=None, data=None,
             file_object=None):
        """Make a POST request to the API.

        Args:
            path: Path of resource URI to where we're making the request.
            headers: Dict of headers to send with request.
            params: Dict with query string parameters.
            data: Body data to send with request.
            file_object: File object with data to send as file.

        Returns:
            A requests response object on success.

        Raises:
            BadRequestError: Request was deemed formatted incorrectly by server.
            UnauthorizedError: API token is incorrect/not valid.
            ForbiddenError: Permission denied.
            APIError: Generic error from requests library.
        """
        url = urljoin(self.__class__.api_base_url, path)
        files = {'file': file_object} if file_object else None
        response = self._request('post', url, headers=headers, params=params,
                                 data=data, files=files)
        return self._check_response(response)

    @requests_exceptions_handling
    def put(self, path, headers=None, params=None, data=None, file_object=None):
        """Make a PUT request to the API.

        Args:
            path: Path of resource URI to where we're making the request.
            headers: Dict of headers to send with request.
            params: Dict with query string parameters.
            data: Body data to send with request.
            file_object: File object with data to send as file.

        Returns:
            A requests response object on success.

        Raises:
            BadRequestError: Request was deemed formatted incorrectly by server.
            UnauthorizedError: API token is incorrect/not valid.
            ForbiddenError: Permission denied.
            APIError: Generic error from requests library.
        """
        url = urljoin(self.__class__.api_base_url, path)
        files = {'file': file_object} if file_object else None
        response = self._request('put', url, headers=headers, params=params,
                                 data=data, files=files)
        return self._check_response(response)

    def _check_response(self, response):
        status_code = response.status_code

        if 399 < status_code and 600 > status_code:
            msg = "%s (%s)" % (response.text, response.url)

            if status_code in self.__class__.error_classes:
                raise self.__class__.error_classes[status_code](
                    msg, response=response)
            elif 399 < response.status_code and 500 > response.status_code:
                raise ClientError(msg, response=response)
            elif 499 < response.status_code and 600 > response.status_code:
                raise ServerError(msg, response=response)

        return response

    def _prepare_requests_kwargs(self, kwargs):
        return kwargs

    def _request(self, method, *args, **kwargs):
        headers = {'user-agent': self.__class__.user_agent,
                   'x-load-impact-agent': self.user_agent}
        if 'headers' not in kwargs:
            kwargs['headers'] = headers
        else:
            if not kwargs['headers']:
                kwargs['headers'] = {}
            kwargs['headers'].update(headers)
        kwargs['timeout'] = self.timeout
        kwargs = self._prepare_requests_kwargs(kwargs)
        return self._requests_request(method, *args, **kwargs)

    def _requests_request(self, method, *args, **kwargs):
        return getattr(requests, method)(*args, **kwargs)


class ApiTokenClient(Client):
    """Client class handling all communication with the Load Impact REST API,
    using simple API token based authentication."""

    def __init__(self, api_token=None, *args, **kwargs):
        super(ApiTokenClient, self).__init__(*args, **kwargs)
        if not api_token:
            try:
                api_token = self._get_api_token_from_environment()
            except KeyError:
                raise MissingApiTokenError("An API token must be specified "
                                           "either as the first argument to "
                                           "ApiClient or by setting the "
                                           "environment variable "
                                           "LOADIMPACT_API_TOKEN.")
        self.api_token = api_token

    def _get_api_token_from_environment(self):
        return os.environ['LOADIMPACT_API_TOKEN']

    def _prepare_requests_kwargs(self, kwargs):
        kwargs['auth'] = (self.api_token, '')
        return kwargs



# coding=utf-8

"""
Copyright 2015 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import absolute_import

__all__ = ['DateTimeField', 'DictField', 'FloatField', 'IntegerField',
           'ListField', 'ObjectField', 'StringField', 'UnicodeField']

import sys

from datetime import datetime
from .exceptions import CoercionError
from .utils import UTC


class Field(object):
    """All field classes derive from this base class."""

    # Option enumeration.
    SERIALIZE = 1

    def __init__(self, value=None, options=None):
        if value is None:
            self._value = self.__class__.default()
        else:
            self._value = self.__class__.coerce(value)
        self.options = []
        if options:
            if isinstance(options, int):
                self.options = [options]
            else:
                self.options = list(options)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        try:
            self._value = self.__class__.coerce(value)
        except CoercionError as e:
            raise ValueError(e)

    def __repr__(self):
        return repr(self._value)

    def has_option(self, option):
        return option in self.options

    @classmethod
    def coerce(cls, value):
        raise NotImplementedError

    @classmethod
    def default(cls):
        return cls.field_type()


class DateTimeField(Field):
    field_type = datetime
    format = '%Y-%m-%dT%H:%M:%S'

    @classmethod
    def coerce(cls, value):
        if not isinstance(value, cls.field_type):
            try:
                return datetime.strptime(value[:-7], cls.format).replace(tzinfo=UTC())

            except ValueError as e:
                raise CoercionError(e)
        return value

    @classmethod
    def default(cls):
        return datetime.utcnow().replace(tzinfo=UTC())


class DictField(Field):
    field_type = dict

    @classmethod
    def coerce(cls, value):
        if not isinstance(value, cls.field_type):
            raise CoercionError("'%s' is not a dict" % repr(value))
        return value


class FloatField(Field):
    field_type = float

    @classmethod
    def coerce(cls, value):
        try:
            return float(value)
        except ValueError as e:
            raise CoercionError(e)


class IntegerField(Field):
    field_type = int

    @classmethod
    def coerce(cls, value):
        try:
            return int(value)
        except ValueError as e:
            raise CoercionError(e)


class ListField(Field):
    field_type = list

    @classmethod
    def coerce(cls, value):
        if not isinstance(value, cls.field_type):
            raise CoercionError("'%s' is not a list" % repr(value))
        return value


class StringField(Field):
    field_type = str

    @classmethod
    def coerce(cls, value):
        try:
            return str(value)
        except ValueError as e:
            raise CoercionError(e)


class UnicodeField(Field):
    field_type = str if sys.version_info >= (3, 0) else unicode

    @classmethod
    def coerce(cls, value):
        try:
            if sys.version_info >= (3, 0):
                return str(value)
            else:
                if isinstance(value, str):
                    return unicode(value, 'utf-8')
                else:
                    return unicode(value)
        except (ValueError, UnicodeDecodeError) as e:
            raise CoercionError(e)


class ObjectField(Field):
    field_type = object

    @classmethod
    def coerce(cls, value):
        try:
            return cls.field_type(value)
        except ValueError as e:
            raise CoercionError(e)


class DataStoreListField(Field):
    field_type = list

    @classmethod
    def coerce(cls, value):
        if not isinstance(value, cls.field_type):
            raise CoercionError("'%s' is not a list" % repr(value))
        r = []
        for ds in value:
            if isinstance(ds, dict) and 'id' in ds:
                r.append(ds['id'])
            elif isinstance(ds, int):
                r.append(ds)
        return r


class BooleanField(Field):
    field_type = bool

    @classmethod
    def coerce(cls, value):
        try:
            return cls.field_type(value)
        except ValueError as e:
            raise CoercionError(e)



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import unittest

try:
    from unittest.mock import MagicMock
except ImportError:
    from mock import MagicMock

from loadimpact3.clients import Client
from loadimpact3.fields import IntegerField
from loadimpact3.resources import (
    DataStore, LoadZone, Resource, UserScenario, UserScenarioValidation)


class MockRequestsResponse(object):
    def __init__(self, status_code=200, **kwargs):
        self.url = 'http://example.com/'
        self.status_code = status_code
        self.text = ''
        self.kwargs = kwargs
        for k, v in kwargs.items():
            setattr(self, k, v)

    def json(self):
        return self.kwargs


class MockClient(Client):
    def __init__(self, response_status_code=200, **kwargs):
        super(MockClient, self).__init__()
        self.response_status_code = response_status_code
        self.kwargs = kwargs
        self.last_request_method = None
        self.last_request_args = None
        self.last_request_kwargs = None

    def _requests_request(self, method, *args, **kwargs):
        self.last_request_method = method
        self.last_request_args = args
        self.last_request_kwargs = kwargs
        if isinstance(kwargs.get('data'), str):
            self.last_request_kwargs['data'] = json.loads(kwargs['data'])
        nkwargs = {}
        if self.kwargs.get('response_body'):
            if isinstance(self.kwargs['response_body'], dict):
                nkwargs = self.kwargs['response_body']
            elif isinstance(self.kwargs['response_body'], str):
                nkwargs = json.loads(self.kwargs['response_body'])
        return MockRequestsResponse(status_code=self.response_status_code,
                                    **nkwargs)


class MockResource(Resource):
    fields = {}
    resource_name = 'resource'

    def __init__(self, client, field_cls, field_value=None):
        self.__class__.fields['field'] = field_cls
        super(MockResource, self).__init__(client, field=field_value)


class TestResourcesResource(unittest.TestCase):
    def test___getattr__(self):
        r = MockResource(None, IntegerField, 0)

        def raises():
            r.field2
        self.assertRaises(AttributeError, raises)

    def test__path(self):
        self.assertEqual(MockResource._path(), MockResource.resource_name)
        self.assertEqual(MockResource._path(resource_id=None),
                         MockResource.resource_name)
        self.assertEqual(MockResource._path(resource_id=0),
                         '%s/%s' % (MockResource.resource_name, 0))
        self.assertEqual(MockResource._path(resource_id=1),
                         '%s/%s' % (MockResource.resource_name, 1))
        self.assertEqual(MockResource._path(resource_id=1, action='action'),
                         '%s/%s/%s' % (MockResource.resource_name, 1, 'action'))


class TestResourcesDataStore(unittest.TestCase):
    def setUp(self):
        self.client = MockClient()

    def test_has_conversion_finished(self):
        ds = DataStore(self.client)
        ds.sync = MagicMock()
        self.assertFalse(ds.has_conversion_finished())

    def test_has_conversion_finished_status_queued(self):
        self._check_has_conversion_finished(DataStore.STATUS_QUEUED, False)

    def test_has_conversion_finished_status_converting(self):
        self._check_has_conversion_finished(DataStore.STATUS_CONVERTING, False)

    def test_has_conversion_finished_status_finished(self):
        self._check_has_conversion_finished(DataStore.STATUS_FINISHED, True)

    def test_has_conversion_finished_status_failed(self):
        self._check_has_conversion_finished(DataStore.STATUS_FAILED, True)

    def _check_has_conversion_finished(self, status, expected):
        ds = DataStore(self.client)
        ds.status = status
        ds.sync = MagicMock()
        self.assertEqual(ds.has_conversion_finished(), expected)


class TestResourcesLoadZone(unittest.TestCase):
    def test_name_to_id(self):
        self.assertEqual(LoadZone.name_to_id(LoadZone.AGGREGATE_WORLD), 1)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_US_ASHBURN), 11)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_US_PALOALTO), 12)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_IE_DUBLIN), 13)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_SG_SINGAPORE), 14)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_JP_TOKYO), 15)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_US_PORTLAND), 22)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_BR_SAOPAULO), 23)
        self.assertEqual(LoadZone.name_to_id(LoadZone.AMAZON_AU_SYDNEY), 25)
        self.assertEqual(LoadZone.name_to_id(LoadZone.RACKSPACE_US_CHICAGO), 26)
        self.assertEqual(LoadZone.name_to_id(LoadZone.RACKSPACE_US_DALLAS), 27)
        self.assertEqual(LoadZone.name_to_id(LoadZone.RACKSPACE_UK_LONDON), 28)
        self.assertEqual(LoadZone.name_to_id(LoadZone.RACKSPACE_AU_SYDNEY), 29)

    def test_name_to_id_exception(self):
        self.assertRaises(ValueError, LoadZone.name_to_id, 'unknown')


class TestResourcesUserScenario(unittest.TestCase):
    def setUp(self):
        self.client = MockClient()

    def test_get(self):
        client = MockClient(response_body={'user_scenario': {
            'id': 1,
            'name': 'My test user scenario'
        }})
        user_scenario = client.get_user_scenario(1)
        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(user_scenario.id, 1)
        self.assertEqual(user_scenario.name, 'My test user scenario')

    def test_update_with_dict(self):
        new_script = '---'
        client = MockClient(response_body={'user_scenario': {'script': new_script}})
        user_scenario = UserScenario(client)
        user_scenario = user_scenario.update_scenario(data={'script': new_script})

        self.assertEqual(client.last_request_method, 'put')
        self.assertEqual(client.last_request_kwargs['data']['script'],
                         new_script)
        self.assertEqual(user_scenario.script, new_script)


class TestResourcesUserScenarioValidation(unittest.TestCase):
    def setUp(self):
        self.client = MockClient()

    def test_is_done(self):
        client = MockClient(response_body={'user_scenario_validation': {
            'id': 1,
            'status': UserScenarioValidation.STATUS_QUEUED
        }})
        validation = client.get_user_scenario_validation(1)
        self.assertFalse(validation.is_done())
        self.assertEqual(client.last_request_method, 'get')

    def test_is_done_status_queued(self):
        self._check_is_done(UserScenarioValidation.STATUS_QUEUED, False)

    def test_is_done_status_initializing(self):
        self._check_is_done(UserScenarioValidation.STATUS_INITIALIZING, False)

    def test_is_done_status_running(self):
        self._check_is_done(UserScenarioValidation.STATUS_RUNNING, False)

    def test_is_done_status_finished(self):
        self._check_is_done(UserScenarioValidation.STATUS_FINISHED, True)

    def test_is_done_status_failed(self):
        self._check_is_done(UserScenarioValidation.STATUS_FAILED, True)

    def test_status_code_to_text(self):
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            UserScenarioValidation.STATUS_QUEUED), 'queued')
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            UserScenarioValidation.STATUS_INITIALIZING), 'initializing')
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            UserScenarioValidation.STATUS_RUNNING), 'running')
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            UserScenarioValidation.STATUS_FINISHED), 'finished')
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            UserScenarioValidation.STATUS_FAILED), 'failed')
        self.assertEqual(UserScenarioValidation.status_code_to_text(
            0xffffffff), 'unknown')

    def _check_is_done(self, status, expected):
        client = MockClient(response_body={'user_scenario_validation': {
            'id': 1
        }})
        validation = client.get_user_scenario_validation(1)
        validation.status = status
        self.assertEqual(validation.is_done(), expected)
        self.assertEqual(client.last_request_method, 'get')


class TestResourcesOrganization(unittest.TestCase):

    def test_list(self):
        client = MockClient(response_body={'organizations': [{
            'id': 1,
            'name': 'My org'
        }]})
        organizations = client.list_organizations()
        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(organizations[0].id, 1)
        self.assertEqual(organizations[0].name, 'My org')


class TestResourcesProject(unittest.TestCase):

    def test_list(self):
        client = MockClient(response_body={'projects': [{
            'id': 1,
            'name': 'My project'
        }]})
        projects = client.list_organization_projects(1)
        self.assertEqual(client.last_request_method, 'get')
        self.assertEqual(projects[0].id, 1)
        self.assertEqual(projects[0].name, 'My project')



# coding=utf-8

"""
Copyright 2013 Load Impact

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# Make sure setup tools is installed, if not install it.
from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup

import os
import sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'loadimpact3'))
from version import __version__

setup(
    name='loadimpact-v3',
    version=__version__,
    author='Load Impact',
    author_email='support@loadimpact.com',
    packages=['loadimpact3'],
    py_modules=['ez_setup'],
    url='http://developers.loadimpact.com/',
    license='LICENSE.txt',
    description="The Load Impact SDK provides access to Load Impact's cloud-based performance testing platform",
    install_requires=['requests', 'six'],
    test_requires=['coverage'],
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    keywords="loadimpact api rest sdk",
    test_suite='test'
)
